// FTP Server demo
// Creates two files, one for anonymous access and accessible only to a
// specified user.
// Enables users to download and upload files.

// This file and other embedded Ethernet and Internet code and resources are
// available from www.Lvr.com.

// Select a network configuration from \lib\tcpip\tcp_config.lib
// in the Dynamic C distribution.
#define TCPCONFIG 1

// Format the file system and reset the list of files in the user block.
// Execute this statement the first time the program runs,
// to put the file system in a known state.
#define FORMAT

// Enables file system 2 (FS2) support in the default functions for the file
// handler and enables clients to write files to the file system.
#define FTP_USE_FS2_HANDLERS

// The maximum number of files the file system can handle.
#define FS_MAX_FILES 50

// Specifies how many kilobytes of program Flash memory the file system can use.
#define FS2_USE_PROGRAM_FLASH 32

// Provides the servermask parameter for the sspec_addfsfile() function,
// which makes files available to the FTP server.
// SERVER_FTP specifies the FTP server.
// SERVER_WRITABLE enables authorized users to delete and overwrite files
// on the server.
#define FTP_CREATE_MASK SERVER_FTP | SERVER_WRITABLE

// A portion of the user block in memory holds a structure that associates
// the names of files with the files locations.
// FTP_USERBLOCK_OFFSET Specifies the offset in the user block where the
// structure will be stored.
// The default is zero. Change this value if the application uses the default
// portion of the user block for another purpose.
#define FTP_USERBLOCK_OFFSET 0

// Specifies the maximum number of files supported by the FTP server.
// The default is 10.
#define SSPEC_MAXSPEC 10

// Must be defined for the FTP server to support the DELE (delete) command.
#define FTP_EXTENSIONS

// All C functions not declared as root go to extended memory.
#memmap xmem

// The dcrtcp library supports IP and TCP.
// The fs2 library supports File System 2.
// The ftp_server library supports FTP server communications.
#use "dcrtcp.lib"
#use "fs2.lib"
#use "ftp_server.lib"

void main() {

	File public_file;
  	File private_file;
  	FSLXnum ext;
	int file;
	int user;
	long len;

   // Sets the maximum size of buffers.
   static char create_file1_buffer[127];
   static char create_file2_buffer[127];

	// fs_get_flash_lx returns a logical extent number that indicates the
   // preferred Flash-memory device for FS2 files.
   // The preferred device is the second Flash memory if one is available, and
   // otherwise is the reserved area in the program Flash memory.

   // To use a portion of the program Flash memory for the file system, you
   // must define two constants. In rabbitbios.c (in the \bios directory of the
   // Dynamic C distribution), set XMEM_RESERVE_SIZE to the number of bytes to
   // reserve for the file system in the program Flash. And in the application,
   // before the statement #use "fs2.lib", define FS2_USE_PROGRAM_FLASH to equal
   // the number of kilobytes the file system will use. The application uses the
   // smaller of the two values. (This application sets FS2_USE_PROGRAM_FLASH to
   // 32 kilobytes above.)

   // To use battery-backed RAM for the file system, use the fs_get_ram_lx()
   // function in place of fs_get_flash_lx().
   // To use the non-preferred Flash memory for the file system, use the
   // fs_get_other_lx() function.

	ext = fs_get_flash_lx();

   // Initialize the file system.
	fs_init(0, 0);

#ifdef FORMAT
	// Put the file system and user block in a known initial state
   // If the FORMAT macro is defined, the file system and user block are
   // initialized to known states.

   // The lx_format() function formats the file system extent, deleting any
   // files that were present.

   // The writeUserBlock() function initializes the user block in memory to
   // zeros. Dont execute this block of code if you want to preserve files
   // already in memory.
	lx_format(ext, 0);
	len = 0;
	writeUserBlock(FTP_USERBLOCK_OFFSET, &len, sizeof(long));
#endif

	// Create an FS2 file and copy the text in create_file1_buffer to the file.
   // The text "public file" is stored in public_file.
   sprintf(create_file1_buffer, "public file");
	fcreate(&public_file, 1);
	fwrite(&public_file, create_file1_buffer, strlen(create_file1_buffer));
	fclose(&public_file);

	// Create an FS2 file and copy the text in create_file2_buffer to the file.
   // The text "private file" is stored in private_file.
   sprintf(create_file2_buffer, "private file");
  	fcreate(&private_file, 2);
	fwrite(&private_file, create_file2_buffer, strlen(create_file2_buffer));
	fclose(&private_file);

	// Load the data structure that keeps track of the locations of the files.
   // On success, ftp_load_filenames() returns zero and a call to
   // ftp_save_filenames() saves the data structure to the user block.
   // Even if there are no file names defined yet, saving the data structure
   // puts it in a known state.
  	if (ftp_load_filenames() < 0) {
		ftp_save_filenames();
	}

	// Set up anonymous access, which doesn't require a password.
	user = sauth_adduser("anonymous", "", SERVER_FTP);
	ftp_set_anonymous(user);

   // All users, including anonymous users, can access public_file as
   // the file "public.txt" on the server.
   file = sspec_addfsfile( "public.txt", 1, SERVER_FTP ) ;
	sspec_setuser(file, user );

	// Set up a user with the user name "rabbit1" and password "embedded".
	user = sauth_adduser("rabbit1", "embedded", SERVER_FTP);

   // Enable write access for the user.
	sauth_setwriteaccess(user, 1);

   // User rabbit1 can access private_file as the file "private.txt" on the
   // server.
   file = sspec_addfsfile( "private.txt", 2, SERVER_FTP ) ;
	sspec_setuser(file, user );

   // Initialize the TCP/IP stack.
	sock_init();

   // Initialize the FTP server using the default handlers.
	ftp_init(NULL);

   // Use the default port (21) for the control channel.
	tcp_reserveport(FTP_CMDPORT);

 	while(1) {

   	// Process FTP requests.
		ftp_tick();
      }

} // end main()